@@ -18,7 +18,7 @@ class single_atom_scatter(object):
         self.composition = composition
         self.q_coords = q_coords
         self.units = units
-        path = os.path.join(os.path.dirname(__file__),'scatteringFactors.txt')
+        path = os.path.join(os.path.dirname(__file__),'scattering_factors.txt')
         self.e_scattering_factors = np.loadtxt(path,dtype=np.float)
 
         return
