@@ -1,3 +1,7 @@
+from .overlay import add_rectangles,add_circles,add_annuli,add_ellipses,add_points, add_grid_overlay
+from .overlay import add_cartesian_grid,add_polarelliptical_grid,add_rtheta_grid,add_scalebar
+from ..io.datastructure import Coordinates
+
 import numpy as np
 import matplotlib.pyplot as plt
 from matplotlib.figure import Figure
@@ -6,14 +10,12 @@ from matplotlib.colors import is_color_like,ListedColormap
 from numpy.ma import MaskedArray
 from numbers import Number
 from math import log
-from .overlay import add_rectangles,add_circles,add_annuli,add_ellipses,add_points
-from .overlay import add_cartesian_grid,add_polarelliptical_grid,add_rtheta_grid,add_scalebar
-from ..io.datastructure import Coordinates
+from copy import copy
 
 def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
          min=None,max=None,power=1,bordercolor=None,borderwidth=5,
          returnclipvals=False,returncax=False,returnfig=False,figax=None,
-         hist=False,n_bins=256,mask=None,mask_color='k',
+         hist=False,n_bins=256,mask=None,mask_color='k',mask_alpha=0.,
          rectangle=None,circle=None,annulus=None,ellipse=None,points=None,grid_overlay=None,
          cartesian_grid=None,polarelliptical_grid=None,rtheta_grid=None,scalebar=None,
          coordinates=None,rx=None,ry=None,space='Q',
@@ -81,7 +83,8 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
         the data array may be passed to the ``mask`` argument, and these pixels will be
         masked.  Masked pixels are displayed as a single uniform color, black by default,
         and which can be specified with the ``mask_color`` argument.  Masked pixels
-        are excluded when displaying the histogram or computing clip values.
+        are excluded when displaying the histogram or computing clip values. The mask
+        can also be blended with the hidden data by setting the ``mask_alpha`` argument.
 
     Overlays (geometric):
         The function natively supports overlaying points, circles, rectangles, annuli,
@@ -219,8 +222,14 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
         mask (None or boolean array): if not None, must have the same shape as 'ar'.
             Wherever mask==True, plot the pixel normally, and where ``mask==False``,
             pixel values are set to mask_color. If hist==True, ignore these values in the
-            histogram
+            histogram. If ``mask_alpha`` is specified, the mask is blended with the array
+            underneath, with 0 yielding an opaque mask and 1 yielding a fully transparent
+            mask. If ``mask_color`` is set to ``'empty'`` instead of a matplotlib.color,
+            nothing is done to pixels where ``mask==False``, allowing overlaying multiple
+            arrays in different regions of an image by invoking the ``figax` kwarg over
+            multiple calls to show
         mask_color (color): see 'mask'
+        mask_alpha (float): see 'mask'
         **kwargs: any keywords accepted by matplotlib's ax.matshow()
 
     Returns:
@@ -231,11 +240,11 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
     assert clipvals in ('minmax','manual','std','centered')
     if mask is not None:
         assert mask.shape == ar.shape
-        assert is_color_like(mask_color)
+        assert is_color_like(mask_color) or mask_color=='empty'
         if isinstance(ar,np.ma.masked_array):
-            ar = np.ma.array(data=ar.data,mask=mask==False)
+            ar = np.ma.array(data=ar.data,mask=np.logical_or(ar.mask,~mask))
         else:
-            ar = np.ma.array(data=ar,mask=mask==False)
+            ar = np.ma.array(data=ar,mask=~mask)
     elif isinstance(ar,np.ma.masked_array):
         pass
     else:
@@ -247,48 +256,46 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
         _ar = ar.copy()
         _mask = np.ones_like(_ar.data,dtype=bool)
     elif scaling == 'log':
-        _mask = ar>0
+        _mask = ar.data>0
         _ar = np.zeros_like(ar.data,dtype=float)
-        _ar[_mask] = np.log(ar[_mask])
-        if min != None:
-            if min > 0:
-                min = np.log(min)
-            else:
-                min = np.min(_ar[_mask])
-        if max != None:
-            max = np.log(max)
+        _ar[_mask] = np.log(ar.data[_mask])
+        _ar[~_mask] = np.nan
+        if clipvals == 'manual':
+            if min != None:
+                if min > 0: min = np.log(min)
+                else: min = np.min(_ar[_mask])
+            if max != None: max = np.log(max)
     elif scaling == 'power':
-        _mask = ar>0
+        _mask = ar.data>0
         _ar = np.zeros_like(ar.data,dtype=float)
-        _ar[_mask] = np.power(ar[_mask],power)
-        if min != None:
-            min = np.power(min,power)
-        if max != None:
-            max = np.power(max,power)
+        _ar[_mask] = np.power(ar.data[_mask],power)
+        _ar[~_mask] = np.nan
+        if clipvals == 'manual':
+            if min != None: min = np.power(min,power)
+            if max != None: max = np.power(max,power)
     else:
         raise Exception
-    _ar = np.ma.array(data=_ar.data,mask=np.logical_and(ar.mask==False,_mask)==False)
+
+    _ar = np.ma.array(data=_ar.data,mask=~_mask)
 
     # Set the clipvalues
     if clipvals == 'minmax':
-        vmin,vmax = np.ma.min(_ar),np.ma.max(_ar)
+        vmin,vmax = np.nanmin(_ar),np.nanmax(_ar)
     elif clipvals == 'manual':
         assert min is not None and max is not None
         vmin,vmax = min,max
     elif clipvals == 'std':
         assert min is not None and max is not None
-        m,s = np.ma.median(_ar),np.ma.std(_ar)
+        m,s = np.nanmedian(_ar),np.nanstd(_ar)
         vmin = m + min*s
         vmax = m + max*s
     elif clipvals == 'centered':
-        c = np.ma.mean(_ar) if min is None else min
-        m = np.ma.max(np.ma.abs(c-_ar)) if max is None else max
+        c = np.nanmean(_ar) if min is None else min
+        m = np.nanmax(np.ma.abs(c-_ar)) if max is None else max
         vmin = c-m
         vmax = c+m
     else:
         raise Exception
-    if returnclipvals:
-        return vmin,vmax
 
     # Create or attach to the appropriate Figure and Axis
     if figax is None:
@@ -298,13 +305,23 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
         assert(isinstance(fig,Figure))
         assert(isinstance(ax,Axes))
 
+    # Create the masked array applying the user mask (this is done after the 
+    # vmin and vmax are determined so the mask doesn't affect those)
+    _ar = np.ma.array(data=_ar.data,mask=np.logical_or(ar.mask,~_mask))
+
+    # Create colormap with mask_color for bad values
+    cm = copy(plt.cm.get_cmap(cmap))
+    if mask_color=='empty':
+        cm.set_bad(alpha=0)
+    else:
+        cm.set_bad(color=mask_color)
+
     # Plot the image
     if not hist:
-        cax = ax.matshow(_ar,vmin=vmin,vmax=vmax,cmap=cmap,**kwargs)
-        if np.any(_ar.mask==True):
-            mask_display = np.ma.array(data=_ar.mask,mask=_ar.mask==False)
-            cmap = ListedColormap([mask_color,'w'])
-            ax.matshow(mask_display,cmap=cmap)
+        cax = ax.matshow(_ar,vmin=vmin,vmax=vmax,cmap=cm,**kwargs)
+        if np.any(_ar.mask):
+            mask_display = np.ma.array(data=_ar.data,mask=~_ar.mask)
+            ax.matshow(mask_display,cmap=cmap,alpha=mask_alpha,vmin=vmin,vmax=vmax)
     # ...or, plot its histogram
     else:
         hist,bin_edges = np.histogram(_ar,bins=np.linspace(np.min(_ar),
@@ -334,7 +351,7 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
     if points is not None:
         add_points(ax,points)
     if grid_overlay is not None:
-        add_grid_overly(ax,grid)
+        add_grid_overlay(ax,grid_overlay)
 
 
     # Parse arguments for scale/coordinate overlays
@@ -449,19 +466,26 @@ def show(ar,figsize=(8,8),cmap='gray',scaling='none',clipvals='minmax',
 
     # Add r-theta grid
     if rtheta_grid is not None:
-        add_rtheta_grid(ax,grid)
-
+        add_rtheta_grid(ax,rtheta_grid)
 
     # Show or return
-    if returnfig:
-        return fig,ax
-    elif returncax:
-        return cax
-    elif figax is not None:
+    returnval = []
+    if returnfig: returnval.append((fig,ax))
+    if returnclipvals:
+        if scaling == 'log':
+            vmin,vmax = np.power(np.e,vmin),np.power(np.e,vmax)
+        elif scaling == 'power':
+            vmin,vmax = np.power(vmin,1/power),np.power(vmax,1/power)
+        returnval.append((vmin,vmax))
+    if returncax: returnval.append(cax)
+    if len(returnval)==0:
+        if figax is None:
+            plt.show()
         return
+    elif(len(returnval))==1:
+        return returnval[0]
     else:
-        plt.show()
-        return
+        return tuple(returnval)
 
 def show_hist(arr, bins=200, vlines=None, vlinecolor='k', vlinestyle='--',
                                         returnhist=False, returnfig=False):
@@ -500,7 +524,7 @@ def show_hist(arr, bins=200, vlines=None, vlinecolor='k', vlinestyle='--',
         plt.show()
         return
     elif returnhist and not returnfig:
-        return counts,bins_edges
+        return counts,bin_edges
     elif not returnhist and returnfig:
         return fig, ax
     else:
